String currentStore = "rootStore";

final Map<String, Map<String, dynamic>> store = {currentStore: {}};

void useRootStore() => useStore('rootStore');

void useStore(String storeName) {
  currentStore = storeName;
  if (!store.containsKey(currentStore)) store[currentStore] = {};
}

void save(String key, value) => store[currentStore][key] = value;

void saveIfNull(String key, value) {
  final curstore = store[currentStore];
  if (curstore.containsKey(key) && curstore[key] != null) {
    return;
  }
  save(key, value);
}

void saveIfNullMultiple(Map<String, dynamic> pairs) {
  pairs.forEach((key, value) {
    saveIfNull(key, value);
  });
}

void saveMultiple(Map<String, dynamic> pairs) {
  pairs.forEach((key, value) {
    save(key, value);
  });
}

Map<String, dynamic> serveStore([String storeName]) {
  storeName ??= currentStore;
  return store[storeName];
}

// Deletes current store or specified store
Map<String, dynamic> removeStore([String storeName]) {
  final Map<String, dynamic> r = store.remove(storeName ?? currentStore);
  if (storeName == null) {
    useRootStore();
  }
  return r;
}

Map<String, dynamic> clearStore([String storeName]) {
  storeName ??= currentStore;
  final Map<String, dynamic> r = store[storeName];
  store[storeName].clear();
  return r;
}

E serve<E>(String key, [defaultvalue]) =>
    (store[currentStore][key] ?? defaultvalue) as E;

List<String> createLogStringList(Map<String, dynamic> store) {
  final List<String> ls = [];
  store.forEach((key, value) {
    ls.add('$key : $value');
  });
  return ls;
}

List<String> pad(List<String> ls, String pad) {
  return ls.map((e) => pad + e).toList();
}

List<String> insertAtEnds(List<String> ls, String begin, String end) {
  ls.insert(0, begin);
  ls.add(end);
  return ls;
}

String listToString(List<String> ls) {
  return ls.reduce((value, element) => value + '\n' + element);
}

void logStore([String storeName]) {
  storeName ??= currentStore;

  final str = listToString(createLogList(storeName, store[storeName]));

  print(str);
}

List<String> createLogList(String storeName, Map<String, dynamic> store) {
  return insertAtEnds(
      pad(createLogStringList(store), '\t'), '*** $storeName', '***');
}

void logCompleteStore() {
  List<String> ls = [];

  store.forEach((key, value) {
    final str =
        listToString((pad(createLogList(key, store[key]), '\t'))..add(''));
    ls.add(str);
  });

  ls.insert(0, '');
  ls = insertAtEnds(ls, '***** Complete Store', '*****');
  print(listToString(ls));
}

// Usage
// main(List<String> args) {
//   useStore('storeName1');
//   save('key', 4);
//   save('key1', 44);
//   save('key2', 444);
//   save('key3', 4444);

//   useStore('a');

//   save('key', 4);
//   save('key1', 44);
//   save('key2', 444);
//   save('key3', 4444);

//   clearStore('storeName1');
//   print(currentStore);
//   logCompleteStore();
// }